package f4.collection;

import java.util.ArrayList;
import java.util.List;

/**
 * Basic usage of a list
 *
 * We prefer Lists to arrays because we get many nice methods to use. Arrays are
 * to primitive (low level)
 *
 * NOTE: There are more implementations of lists but they all work the same from
 * our perspective (fulfilling the List contract)
 *
 * // Possible List<String> l1 = new ArayList<>(); // Or like this List<String>
 * l2 = new LinkedList<>();
 *
 * @author hajo
 */
public class UseAList {

    public static void main(String args[]) {
        // List is the abstract specification of a list (an interface)
        // ArrayList is a class fulfilling the specification
        // We *always* do like this
        List<String> ss = new ArrayList<>();  // Must specify what's in the list (Stings)

        // List is empty after creation
        System.out.println(ss.size());
        System.out.println(ss.isEmpty());

        // Add to the end of the list 
        ss.add("first");
        ss.add("second");
        String trd = "another";
        ss.add(trd);
        ss.add(trd);  // Add a duplicate
        System.out.println(ss);
        System.out.println("Size is " + ss.size());

        // Add at index (other elements shifted right)
        ss.add(0, "reallyFirst");
        System.out.println(ss);
        //ss.add(7, "badNoIndex7");   // Exception

        // Find index for
        int i = ss.indexOf(trd);
        System.out.println("Found at " + i);
        int j = ss.lastIndexOf(trd);
        System.out.println("Found at " + j);

        // Find object by index
        String fst = ss.get(0);
        System.out.println("At index 0 is " + fst);

        // Travering (run through) list
        for (String s : ss) {
            System.out.println("*" + s);
        }

        // Replace (overwrite)
        ss.set(2, "Overwritten");
        System.out.println(ss);
        //ss.set(7, "badNoIndex7");

        // Remove by index (others shifted left)
        String removed = ss.remove(1);
        System.out.println(removed + ":" + ss);
        // Remove by object (first from left if more)
        boolean ok = ss.remove(trd);
        System.out.println(ok + ":" + ss);

        // Remove from end. Will get a new list, must assign else sublist lost!!!
        ss = ss.subList(0, ss.size() - 1);
        System.out.println(ss);

        // Is this allowed?
        ss.add(null);
        ss.add(null);
        System.out.println("Added null " + ss); // Yes, but we shouldn't do this!!!
        ss.remove(null);
        System.out.println("Removed null " + ss); // Yes, but we shouldn't do this!!!
       
        // Remove all
        ss.clear();
        System.out.println("Size is " + ss.size());

        // Add a duplicate
        String s1 = "theString";
        String s2 = s1;
        ss.add(s1);
        ss.add(s2);
        System.out.println("Size with duplicates is " + ss.size());
        ss.remove(s1);
        System.out.println("Size with duplicates and remove is " + ss.size());
        System.out.println(ss);

    }
}
