package f4.string;

/**
 * Sometimes immutable (unchangeable) objects are to costly to use. 
 * Example shows String (immutable) vs StringBuilder (mutable)
 *
 * @author hajo
 *
 */
public class StringProducer {

    private final int manyMany = 1000;

    // String is immutable, new objects
    // created and values (chars) copied from old
    // DON't do this
    public String horrible() {
        String s = "";
        for (int i = 0; i < manyMany; i++) {
            s += getLongString(); // + Very slow
        }
        return s;
    }

    // Much quicker
    public String theWayToDoIt() {
        StringBuilder b = new StringBuilder(manyMany * 30);
        for (int i = 0; i < manyMany; i++) {
            b.append(getLongString());
        }
        return b.toString(); // Return immutable
    }

    private String getLongString() {
        return "qwertyuiopasdfghjklmnzxcvbnm";
    }
}

